#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <Hash.h>
#include <ESPAsyncTCP.h>
#include <ESPAsyncWebServer.h>
#include <ESP8266WebServer.h>
#include <WiFiManager.h>         //https://github.com/tzapu/WiFiManager
//#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include <ESP_Mail_Client.h>

void connectWIFI(void);
void setupmenu(void);
void readPROM(void);
void writePROM(void);
void notFound(AsyncServerRequest *request);

String ssid;
String pass;
String email;
String text1;
String text2;
char ssidbuffer[20];
char passbuffer[20];
char emailbuffer[40];

#define WIFI_SSID "ssid"
#define WIFI_PASSWORD "password"

#define SMTP_HOST "smtp.gmail.com"    // Google
//#define SMTP_HOST "mail.gmx.de"         // GMX
//#define SMTP_HOST "smtp.web.de"         // Web.de
#define SMTP_PORT 465 //465, 465, 587

/* The sign in credentials */
#define AUTHOR_EMAIL "xy@gmail.com"
//#define AUTHOR_EMAIL "xy@gmx.de"
//#define AUTHOR_EMAIL "xy@web.de"
#define AUTHOR_PASSWORD "password"
//#define AUTHOR_PASSWORD "password"

/* Recipient's email*/
#define RECIPIENT_EMAIL "xy.gmx.de"

// Default Recipient Email Address
String inputMessage = "xy@gmx.de";
String enableEmailChecked = "checked";
String inputMessage2 = "true";
// Default Threshold Temperature Value
String inputMessage3 = "25.0";
String lastTemperature;

#define BUTTON_ALARM 0
#define BUTTON_FENCE 4

const char* PARAM_INPUT_1 = "email_input";
const char* PARAM_INPUT_2 = "enable_email_input";
const char* PARAM_INPUT_3 = "threshold_input";

/* The SMTP Session object used for Email sending */
SMTPSession smtp;

/* Declare the session config data */
ESP_Mail_Session session;
/* Declare the message class */
SMTP_Message message;
/* Callback function to get the Email sending status */
void smtpCallback(SMTP_Status status);

// Create AsyncWebServer object on port 80
//AsyncWebServer server(80);
AsyncServer server(80);

// HTML web page to handle 3 input fields (email_input, enable_email_input, threshold_input)
const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html><head>
  <title>Email Notification with Temperature</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  </head><body>
  <h2>DS18B20 Temperature</h2> 
  <h3>%TEMPERATURE% &deg;C</h3>
  <h2>ESP Email Notification</h2>
  <form action="/get">
    Email Address <input type="email" name="email_input" value="%EMAIL_INPUT%" required><br>
    Enable Email Notification <input type="checkbox" name="enable_email_input" value="true" %ENABLE_EMAIL%><br>
    Temperature Threshold <input type="number" step="0.1" name="threshold_input" value="%THRESHOLD%" required><br>
    <input type="submit" value="Submit">
  </form>
</body></html>)rawliteral";

void notFound(AsyncServerRequest *request) {
  request->send(404, "text/plain", "Not found");
}

// Replaces placeholder with DS18B20 values
String processor(const String& var){
  //Serial.println(var);
  if(var == "TEMPERATURE"){
    return lastTemperature;
  }
  else if(var == "EMAIL_INPUT"){
    return inputMessage;
  }
  else if(var == "ENABLE_EMAIL"){
    return enableEmailChecked;
  }
  else if(var == "THRESHOLD"){
    return inputMessage3;
  }
  return String();
}


void setup(){
  Serial.begin(115200);
  delay(100);
  Serial.println("**************");
  Serial.println("ESP07_Alarm_06_2");
  Serial.println("**************");
  Serial.println();

  pinMode(BUTTON_ALARM,INPUT);
  pinMode(BUTTON_FENCE,INPUT);
  
  readPROM();
  Serial.println("") ;
  Serial.print("ssid: "); Serial.println(ssid);
  Serial.print("password: "); Serial.println(pass);
  Serial.print("E-Mail Address: "); Serial.println(email);
  Serial.print("Subject: "); Serial.println(text1);
  Serial.print("Message: "); Serial.println(text2);
  Serial.println("Press any Key to start Setup");
  Serial.println("Continue with: '*' ");

  //WiFiManager
    //Local intialization. Once its business is done, there is no need to keep it around
    WiFiManager wifiManager;
    //reset saved settings
    wifiManager.resetSettings();
    
    //set custom ip for portal
    //wifiManager.setAPStaticIPConfig(IPAddress(10,0,1,1), IPAddress(10,0,1,1), IPAddress(255,255,255,0));

    //fetches ssid and pass from eeprom and tries to connect
    //if it does not connect it starts an access point with the specified name
    //here  "AutoConnectAP"
    //and goes into a blocking loop awaiting configuration
    wifiManager.autoConnect("AutoConnectAP");
    //or use this for auto generated name ESP + ChipID
    //wifiManager.autoConnect();

    
    //if you get here you have connected to the WiFi
    Serial.println("connected...yeey :)");
     
  /** Enable the debug via Serial port
   * none debug or 0
   * basic debug or 1
  */
  smtp.debug(1);

  /* Set the callback function to get the sending results */
  smtp.callback(smtpCallback);

  /* Declare the session config data */
  //ESP_Mail_Session session;

  /* Set the session config */
  session.server.host_name = SMTP_HOST;
  session.server.port = SMTP_PORT;
  session.login.email = AUTHOR_EMAIL;
  session.login.password = AUTHOR_PASSWORD;
  session.login.user_domain = "";

  /* Declare the message class */
  //SMTP_Message message;

  /* Set the message headers */
  message.sender.name = "ESP";
  message.sender.email = AUTHOR_EMAIL;
  message.subject = "Alert: Renosterkop R13a";
  message.addRecipient("Juergen Babenhauserheide", email.c_str());

  /*Send HTML message*/
  String htmlMsg = "<div style=\"color:#2f4468;\"><h1>Alert</h1><p>- Help, we are attacked</p></div>";
  message.html.content = htmlMsg.c_str();
  message.html.content = htmlMsg.c_str();
  message.text.charSet = "us-ascii";
  message.html.transfer_encoding = Content_Transfer_Encoding::enc_7bit;

  /*
  //Send raw text message
  String textMsg = "Hello World! - Sent from ESP board";
  message.text.content = textMsg.c_str();
  message.text.charSet = "us-ascii";
  message.text.transfer_encoding = Content_Transfer_Encoding::enc_7bit;
  
  message.priority = esp_mail_smtp_priority::esp_mail_smtp_priority_low;
  message.response.notify = esp_mail_smtp_notify_success | esp_mail_smtp_notify_failure | esp_mail_smtp_notify_delay;*/

  /* Set the custom message header */
  //message.addHeader("Message-ID: <abcde.fghij@gmail.com>");

  /* Connect to server with the session config */
  //if (!smtp.connect(&session))
  //  return;

  /* Start sending Email and close the session */
  //if (!MailClient.sendMail(&smtp, &message))
  //  Serial.println("Error sending Email, " + smtp.errorReason());

  // Send web page to client
  server.on("/", HTTP_GET, [](AsyncServerRequest *request){
    request->send_P(200, "text/html", index_html, processor);
  });

  // Receive an HTTP GET request at <ESP_IP>/get?email_input=<inputMessage>&enable_email_input=<inputMessage2>&threshold_input=<inputMessage3>
  server.on("/get", HTTP_GET, [] (AsyncServerRequest *request) {
    // GET email_input value on <ESP_IP>/get?email_input=<inputMessage>
    if (request->hasParam(PARAM_INPUT_1)) {
      inputMessage = request->getParam(PARAM_INPUT_1)->value();
      // GET enable_email_input value on <ESP_IP>/get?enable_email_input=<inputMessage2>
      if (request->hasParam(PARAM_INPUT_2)) {
        inputMessage2 = request->getParam(PARAM_INPUT_2)->value();
        enableEmailChecked = "checked";
      }
      else {
        inputMessage2 = "false";
        enableEmailChecked = "";
      }
      // GET threshold_input value on <ESP_IP>/get?threshold_input=<inputMessage3>
      if (request->hasParam(PARAM_INPUT_3)) {
        inputMessage3 = request->getParam(PARAM_INPUT_3)->value();
      }
    }
    else {
      inputMessage = "No message sent";
    }
    Serial.println(inputMessage);
    Serial.println(inputMessage2);
    Serial.println(inputMessage3);
    request->send(200, "text/html", "HTTP GET request sent to your ESP.<br><a href=\"/\">Return to Home Page</a>");
  });
  server.onNotFound(notFound);
  server.begin();
  
}

void loop(){
  
   if(digitalRead(BUTTON_FENCE) == 0) {
  /* Connect to server with the session config */
  if (!smtp.connect(&session))
    return;

  /* Start sending Email and close the session */
  if (!MailClient.sendMail(&smtp, &message))
    Serial.println("Error sending Email, " + smtp.errorReason());
   }  
}

/* Callback function to get the Email sending status */
void smtpCallback(SMTP_Status status){
  /* Print the current status */
  Serial.println(status.info());

  /* Print the sending result */
  if (status.success()){
    Serial.println("----------------");
    ESP_MAIL_PRINTF("Message sent success: %d\n", status.completedCount());
    ESP_MAIL_PRINTF("Message sent failled: %d\n", status.failedCount());
    Serial.println("----------------\n");
    struct tm dt;

    for (size_t i = 0; i < smtp.sendingResult.size(); i++){
      /* Get the result item */
      SMTP_Result result = smtp.sendingResult.getItem(i);
      time_t ts = (time_t)result.timestamp;
      localtime_r(&ts, &dt);

      ESP_MAIL_PRINTF("Message No: %d\n", i + 1);
      ESP_MAIL_PRINTF("Status: %s\n", result.completed ? "success" : "failed");
      ESP_MAIL_PRINTF("Date/Time: %d/%d/%d %d:%d:%d\n", dt.tm_year + 1900, dt.tm_mon + 1, dt.tm_mday, dt.tm_hour, dt.tm_min, dt.tm_sec);
      ESP_MAIL_PRINTF("Recipient: %s\n", result.recipients);
      ESP_MAIL_PRINTF("Subject: %s\n", result.subject);
    }
    Serial.println("----------------\n");
  }
}

void connectWIFI() {
  readPROM();
  Serial.println("") ;
  Serial.println("Connect to ") ;
  Serial.println(ssid) ;
  Serial.println(pass) ;
  ssid.toCharArray(ssidbuffer,20);
  pass.toCharArray(passbuffer,20);
  //WiFi.begin(ssidbuffer,passbuffer );
  //while (WiFi.status()!=WL_CONNECTED) {
  //  delay(500);
  //  Serial.print(".");
  //}
  Serial.println("");
  Serial.print("Connected ");
  Serial.println(ssid);
  //Serial.print("IP address: ");
  //Serial.println(WiFi.localIP());
}

void setupmenu() {
  Serial.println("Enter Setup");
  Serial.print("SSID:");
  ssid=String("");
  while(true) {
    if (Serial.available()) {
      char c=Serial.read();
      Serial.write(c);
      if (c=='*') {
        break;
      }
      else {
        ssid=String(ssid+c);
      }
    yield();
    }
  }
  Serial.print("Password:") ;
  pass=String("");
  while(true) {
    if (Serial.available()) {
      char c=Serial.read();
      Serial.write(c);
      if (c=='*') {
        break;
      }
      else {
        pass=String(pass+c);
      }
      yield();
    }
  }   
 // Eingabe E-Mail Adresse
 Serial.print("E-Mail Address:") ;
  email=String("");
  while(true) {
    if (Serial.available()) {
      char c=Serial.read();
      Serial.write(c);
      if (c=='*') {
        break;
      }
      else {
        email=String(email+c);
      }
      yield();
    }
  }  
  // Eingabe Subject

  // Eingabe Nachricht
  
  ssid.trim();
  pass.trim();
  email.trim();
  Serial.println("");
  Serial.println(ssid);
  Serial.println("");
  Serial.println(pass);
  Serial.println("");
  Serial.println(email);
  writePROM();
  Serial.println("Saved to EEPROM");
  }

void readPROM() {
  EEPROM.begin(512);
  EEPROM.get(0,ssidbuffer);
  ssid=String(ssidbuffer);
  EEPROM.get(21,passbuffer);
  pass=String(passbuffer);
  EEPROM.get(42,emailbuffer);
  email=String(emailbuffer);
  EEPROM.commit();
  EEPROM.end();
}

void writePROM() {
  EEPROM.begin(512);
  strncpy(ssidbuffer,ssid.c_str(),20);
  EEPROM.put(0,ssidbuffer);
  strncpy(passbuffer,pass.c_str(),20);
  EEPROM.put(21,passbuffer);
  strncpy(emailbuffer,email.c_str(),40);
  EEPROM.put(42,emailbuffer);
  EEPROM.commit();
  EEPROM.end();
}
